import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

public class TestGen {
    Random random = new Random(0xAAA);

    int testCount;
    public static final int MAX_N = 16;

    void print(int n, int[] x, int[] y, int[] z, String comment) throws IOException {
        if (n > MAX_N)
            throw new IllegalArgumentException("Too big n");
        if (x.length > n * n * n)
            throw new IllegalArgumentException("Too long array");

        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println("Generating test #" + testCount + ": " + comment);
        out.println(n + " " + x.length);
        for (int i = 0; i < x.length; ++i)
            out.println((x[i] + 1) + " " + (y[i] + 1) + " " +  (z[i] + 1));
        out.close();
    }

    int[] toArr(List<Integer> list) {
        int[] a = new int[list.size()];
        for (int i = 0; i < list.size(); ++i)
            a[i] = list.get(i);
        return a;
    }

    void print(int n, List<Integer> x, List<Integer> y, List<Integer> z, String comment) throws IOException {
        print(n, toArr(x), toArr(y), toArr(z), comment);
    }

    void genSamples() throws IOException {
        print(5, new int[]{0}, new int[]{1}, new int[]{2}, "sample1");
        print(6, new int[]{3, 1, 1, 3, 0}, new int[]{0, 2, 3, 4, 1}, new int[]{1, 4, 4, 5, 5}, "sample2");
        print(3, new int[]{}, new int[]{}, new int[]{}, "sample3");
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();
        genHandleTests();
        genFull(MAX_N, 0, "full 0");
        genFull(MAX_N, 1, "full 1");
        genFull(MAX_N, 1, "full 1");
        genFull(MAX_N, 10, "full 10");
        genFull(MAX_N - 1, MAX_N, "full max_n");
        genFull(MAX_N - 2, 2 * MAX_N, "full 2*max_n");
        genFull(MAX_N, 3 * MAX_N, "full 3*max_n");
        for (int i = 1; i <= MAX_N / 2; ++i) {
            genByNumComp(MAX_N - random.nextInt(3), i, 0);
            genByNumComp(MAX_N - random.nextInt(3), i, 1);
        }
        for (int i = 0; i < 10; ++i) {
            int x = random.nextInt(MAX_N - 2) + 3;
            int y = random.nextInt(x / 2) + 1;
            genByNumComp(x, y, 1);
        }
        genRandTriplets(MAX_N, 1);
        genRandTriplets(MAX_N, 2);
        genRandTriplets(MAX_N, 3);
        genRandTriplets(MAX_N, MAX_N);
    }

    private void genRandTriplets(int n, int m) throws IOException{
        List<Integer> x = new ArrayList<>();
        List<Integer> y = new ArrayList<>();
        List<Integer> z = new ArrayList<>();
        for (int i = 0; i < m; ++i) {
            int xi = random.nextInt(n);
            int yi = random.nextInt(n);
            int zi = random.nextInt(n);
            while (xi == yi || yi == zi) {
                xi = random.nextInt(n);
                yi = random.nextInt(n);
                zi = random.nextInt(n);
            }
            x.add(xi);
            y.add(yi);
            z.add(zi);
        }
        print(n, x, y, z, "genRandTriplets m = " + m);
    }

    private void genByNumComp(int n, int comp, int tp) throws IOException {
        if (tp == 0) {
            int[] a = new int[n];
            List<Integer> p = randPerm(n);
            for (int i = 0; i < n; ++i)
                a[p.get(i)] = i % comp;
            genByArray(a, "genByNumComp n=" + n + ", comp=" + comp + ", uniformly");
        } else {
            List<Integer> perm = randPerm(n - 1);
            List<Integer> comb = new ArrayList<>();
            for (int i = 0; i < comp - 1; ++i)
                comb.add(perm.get(i));
            Collections.sort(comb);
            for (int i = 0; i + 1 < comb.size(); ++i)
                if (comb.get(i) + 1 == comb.get(i + 1))
                    for (int j = i + 1; j < comb.size(); ++j)
                        comb.set(j, comb.get(j) + 1);
            List<Integer> p = randPerm(n);
            int j = 0;
            int[] a = new int[n];
            for (int i = 0; i < n; ++i) {
                a[p.get(i)] = j;
                if (j < comb.size() && comb.get(j) == i) ++j;
            }
            genByArray(a, "genByNumComp n=" + n + ", compInit = " + comp + ", comp=" + (j + 1));
        }
    }

    private void genByArray(int[] a, String comment) throws IOException {
        genByArray(a, 0.2, comment);
    }

    private void genByArray(int[] a, double part, String comment) throws IOException {
        int n = a.length;
        List<Integer> x = new ArrayList<>();
        List<Integer> y = new ArrayList<>();
        List<Integer> z = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j)
                for (int k = 0; k < n; ++k)
                    if (i != j && j != k && i != k && !(a[i] == a[j] && a[i] == a[k])) {
                        x.add(i);
                        y.add(j);
                        z.add(k);
                    }
        int k = (int)(x.size() * (part * random.nextDouble()));
        List<Integer> p = randPerm(x.size());
        p = p.subList(0, k);
        Collections.sort(p, Comparator.<Integer>reverseOrder());
        for (int i = 0; i < k; ++i) {
            x.remove((int)p.get(i));
            y.remove((int)p.get(i));
            z.remove((int)p.get(i));
        }
        print(n, x, y, z, comment);
    }

    List<Integer> randPerm(int n) {
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < n; ++i) list.add(i);
        Collections.shuffle(list, random);
        return list;
    }

    private void genFull(int n, int rem, String comment) throws IOException {
        List<Integer> x = new ArrayList<>();
        List<Integer> y = new ArrayList<>();
        List<Integer> z = new ArrayList<>();
        if (random.nextBoolean())
            for (int i = 0; i < n; ++i)
                for (int j = i + 1; j < n; ++j)
                    for (int k = j + 1; k < n; ++k) {
                        x.add(i);
                        y.add(j);
                        z.add(k);
                    }
        else
            for (int i = 0; i < n; ++i)
                for (int j = 0; j < n; ++j)
                    for (int k = 0; k < n; ++k)
                        if (i != j && j != k && i != k) {
                            x.add(i);
                            y.add(j);
                            z.add(k);
                        }

        for (int i = 0; i < rem; ++i) {
            x.remove(x.size() - 1);
            y.remove(y.size() - 1);
            z.remove(z.size() - 1);
        }
        print(n, x, y, z, comment);
    }

    private void genHandleTests() throws IOException {
        print(1, new int[]{}, new int[]{}, new int[]{}, "n=1");
        print(2, new int[]{}, new int[]{}, new int[]{}, "n=2");
        print(3, new int[]{}, new int[]{}, new int[]{}, "n=3, m = 0");
        print(MAX_N, new int[]{}, new int[]{}, new int[]{}, "n=" + MAX_N + ", m = 0");
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
